// test.c
#define F_CPU 16000000


// shift registers
#define ds PD0
#define st_cp PD1
#define sh_cp PD2 
#define dsHigh() (PORTD |= (1<<ds))
#define dsLow() (PORTD &= ~(1<<ds))

#include <avr/io.h>
#include <util/delay.h>
#include "tcs230.h"

int dnum = 0;

uint8_t digit(int num) {
	uint8_t dig = 0b01000000;
	switch(num % 10) {
		case 0:
			break;
		case 1:
			dig = 0b01111001;
			break;
		case 2:
			dig = 0b00100100;
			break;
		case 3:
			dig = 0b00110000;
			break;
		case 4:
			dig = 0b00011001;
			break;
		case 5:
			dig = 0b00010010;
			break;
		case 6:
			dig = 0b00000010;
			break;
		case 7:
			dig = 0b01111000;
			break;
		case 8:
			dig = 0b00000000;
			break;
		case 9:
			dig = 0b00010000;
			break;
	}
	return dig;
}

// init shift register
void initSR() {
	DDRD |= (1<<sh_cp)|(1<<st_cp)|(1<<ds);
}

void initServo() {
	TCCR1A|=(1<<COM1A1)|(1<<COM1B1)|(1<<WGM11);        //NON Inverted PWM
	TCCR1B|=(1<<WGM13)|(1<<WGM12)|(1<<CS11)|(1<<CS10); //PRESCALER=64 MODE 14(FAST PWM)
	
	TCNT1 = 0;
	ICR1=4999;  //fPWM=50Hz (Period = 20ms Standard).

	DDRD |= (1 << PD4) | (1 << PD5);   //PWM Pins as Out
	//OCR1A = 700;
	//OCR1B = 310;
   
}

void pulse() {
	PORTD |= (1<<sh_cp); // high
	PORTD &= ~(1<<sh_cp); // low
}

void latch() {
	PORTD |= (1<<st_cp);
	_delay_loop_1(1);
	PORTD &= ~(1<<st_cp);
	_delay_loop_1(1);
}

void write(uint8_t data) {
	//PORTD |= (1<<st_cp);
	for (uint8_t i = 0; i < 8; ++i)
	{
		if(data & 0b10000000) {
			dsHigh();
		} else {
			dsLow();
		}
		pulse();
		latch();
		data = data<<1;
	}
	
	//PORTD &= ~(1<<st_cp);
}

void wait(int dig1, int dig2, int dig3, int dig4, 
	int dig5, int dig6, int dig7, int dig8) {
	uint8_t data2 = 0b10000000;
	for (uint8_t i = 0; i < 60; ++i)
	{
		_delay_ms(5);
		data2 = 0b10001000;
    	write(data2);
		write(digit(dig1));
		write(digit(dig5));
		_delay_ms(5);
		data2 = 0b01000100;
    	write(data2);
		write(digit(dig2)-0b10000000);
		write(digit(dig6));
		_delay_ms(5);
		data2 = 0b00100010;
    	write(data2);
		write(digit(dig3));
		write(digit(dig7));
		_delay_ms(5);
		data2 = 0b00010001;
    	write(data2);
		write(digit(dig4));
		write(digit(dig8));
		_delay_ms(5);
	}
}

unsigned int MeasureR()
{
	TCSSelectRed();
	unsigned int r;
	
	_delay_ms(10);
	r=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();
	
	return r/3.3;
}
unsigned int MeasureG()
{
	TCSSelectGreen();
	unsigned int r;
	
	_delay_ms(10);
	r=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();
	
	return r/3;
}
unsigned int MeasureB()
{
	TCSSelectBlue();
	unsigned int r;
	
	_delay_ms(10);
	r=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();
	
	return r/4.2;
}
unsigned int MeasureC()
{
	TCSSelectClear();
	unsigned int r;
	
	_delay_ms(10);
	r=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();	
	
	_delay_ms(10);
	r+=TCSMeasure();
	
	return r/3;	
}

void sort() {
	unsigned int r2, g2, b2;
	unsigned int r, g, b;
	unsigned int v1, v2, d;
	int count = 0;
	int servo_angle = 130;

	while (1) {
		TCSLEDOn();
		v1=MeasureC();
		
		_delay_ms(100);
		
		v2=MeasureC();
		TCSLEDOff();
		d=v1-v2;
		
		if(d>8000)
		{
			TCSLEDOn();
			
			r = MeasureR();
			g = MeasureG();
			b = MeasureB();
			
			TCSLEDOff();
			
			unsigned int smallest;
		
			if(r < b)
			{
				if(r < g)
					smallest = r;
				else
					smallest = g;
			}
			else
			{
				if(b < g)
					smallest = b;
				else
					smallest = g;	
			}
			
			smallest = smallest / 10;
			
			r2 = r / smallest;
			g2 = g / smallest;
			b2 = b / smallest;
			
			if (r2 == 10 && g2 == 10 && b2 == 10 && smallest > 200) {
				// White /
				servo_angle = 0;
			}
			else if (r2 > b2 && r2 > g2 && b2 <= 12 && g2 <= 12 && r2 < 15) {
				// Brown /
				servo_angle = 170;
			}
			else if (r2 > b2 && r2 > g2 && b2 <= 12 && g2 <= 12) {
				// Red /
				servo_angle = 210;
			}
			else if (r2 > b2 && g2 > b2 && r2 > g2 && b2 <= 12 && g2 < 15) {
				// Orange /
				servo_angle = 250;
			}
			else if (r2 > b2 && g2 > b2 && r2 > g2 && b2 <= 12) {
				// Yellow /
				servo_angle = 310;
			}
			else if (g2 > b2 && g2 >= r2 && b2 <= 12) {
				// Green /
				servo_angle = 370;
				
			}
			else if (b2 > g2 && b2 >= 15 && r2 <= 12) {
				// Blue /
				servo_angle = 410;
			}
			
					
			OCR1A = 316;
			OCR1B = servo_angle;
			
			count = 0;

			//_delay_ms(300);
		} else {
			count++;	
		}
		
		if (count == 5 || count == 0) {
			OCR1A = 525;
			count = 0;
		}
		
		if (servo_angle == 410) {
			PORTD |= (1<<PD7);
			_delay_ms(1500);
			PORTD &= ~(1<<PD7);
		}
		//_delay_ms(100);
	}
	
}
int main() {
	

	initSR();
	//RGB 
	//DDRD |= (1<<PC0) | (1<<PC1) | (1<<PC2); // output

    DDRD |= (1 << PD7); // led

    // senzor culoare
    InitTCS3200();

    int ch = -1;

    // afisaj 7 segmente
    int dig1, dig2, dig3, dig4, dig5, dig6, dig7, dig8;
    dig1 = 0;
    dig2 = 0;
    dig3 = 0;
    dig4 = 0;
    dig5 = 0;
    dig6 = 0;
    dig7 = 0;
    dig8 = 0;

    TCSSetFrequency();
    initServo();

    /*DDRB |= (1<<PB0) | (1<<PB1);

    //20
    PORTB |= (1<<PB0);
    PORTB &= ~(1<<PB1);*/

    int angle = 0;
    int a[6];
    int index = 0;;
    
    a[0] = 100;
    a[1] = 300;
    a[2] = -200;
    a[3] = 300;
    a[4] = -200;
    a[5] = 300;
    while(1) {

    	// test 
     	//sort();
     	TCSLEDOn();

     	if(index > 5) {
     		index = 0;
     		angle = 0;
     	}
     	angle += a[index];

     	OCR1A = 130;
     	wait(dig1,dig2,dig3,dig4, dig5, dig6, dig7, dig8);
		//_delay_ms(1500);
		OCR1A = 365;
		wait(dig1,dig2,dig3,dig4, dig5, dig6, dig7, dig8);
		//_delay_ms(1500);
		OCR1B = angle;
		wait(dig1,dig2,dig3,dig4, dig5, dig6, dig7, dig8);
     	//_delay_ms(1500);
		OCR1A = 700;
		wait(dig1,dig2,dig3,dig4, dig5, dig6, dig7, dig8);
		//_delay_ms(1500);

		if(index == 0) {
			dig1 += 1;
		} if(index == 1) {
			dig4 += 1;
		} if(index == 2) {
			dig2 += 1;
		} if(index == 3) {
			dig5 += 1;
		} if(index == 4) {
			dig3 += 1;
		} if(index == 5) {
			dig6 += 1;
		} 

		int sum = dig1 + dig2 + dig3 + dig4 + dig5 + dig6;
		dig8 = (sum/10)%10;
		dig7 = sum%10;
     	index += 1;
		

		// afisaj 7 segmnete
     	/*dig1 = dnum%10;
     	dig2 = (dnum/10)%10;
     	dig3 = (dnum/100)%10;
     	dig4 = (dnum/1000)%10;
     	dig5 = (dnum/10000)%10;
     	dig6 = (dnum/100000)%10;
     	dig7 = (dnum/1000000)%10;
     	dig8 = (dnum/10000000)%10;*/
        
    }
    return 0;
}
/*
     	// servo
        PORTB |= (1 << PB4);
        for (int i = 0; i < n2; ++i)
        {
        	_delay_us(100);
        }
		
		PORTB &= ~(1 << PB4);
		_delay_ms(5);
		n2 += 2;
		if(n2 > 30)
			n2 = 0;

		// servo motor
		if(ch%10 == 0) {
			PORTB |= (1 << PB3);
	        for (int i = 0; i < 30; ++i)
	        {
	        	_delay_us(100);
	        }
			
			PORTB &= ~(1 << PB3);
			_delay_ms(10);
		}*/
        

    	// senzor culoare
     	/*PORTA |= (1<<PA2);
     	PORTA |= (1<<PA3);

     	PORTA &= ~(1<<PA4);
     	if(PINA & (1 << PA4) == 0) {
     		PORTD |= (1 << PD7);
     		PORTA |= (1<<PA2);
     	}*/